<template>
  <ModalDialog
    ref="updateModal"
    is-page
    size="2xl"
    @close="$router.push({ name: 'links' })"
  >
    <ModalHeader>
      <ModalTitle>{{ $t("app.links.update.title") }}</ModalTitle>
    </ModalHeader>
    <ModalForm
      :is-loading="linkFetch.isFetching"
      @submit="submit"
    >
      <template #default>
        <AlertError v-if="form.errors && form.errors?.unknown">
          {{ form.errors.unknown }}
        </AlertError>
        <template v-if="linkFetch.hasErrors">
          <ErrorState />
        </template>
        <template v-else>
          <FormInput
            id="url"
            :errors="form.errors.url"
          >
            <FormLabel>{{ $t("app.links.form.url") }}</FormLabel>
            <FormText
              v-model="form.values.url"
              name="url"
              required
              type="url"
            />
          </FormInput>
          <LinksOptionsForm
            v-model="form.values"
            :errors="form.errors"
          />
        </template>
      </template>
      <template #footer>
        <FormSubmit
          :processing="form.isFetching"
          class="btn btn-black"
        >
          {{ $t("app.links.update.submit") }}
        </FormSubmit>
        <LinkDeleteModal
          v-if="hasPermission(Permission.LINKS_DELETE)"
          :id="String($route.params.id)"
          v-slot="{ open }"
          @confirm="onDeleted"
        >
          <button
            class="btn btn-outline-red"
            type="button"
            @click="open"
          >
            <ButtonIcon :icon="TrashCanRegularIcon">
              {{ $t("app.links.delete.title") }}
            </ButtonIcon>
          </button>
        </LinkDeleteModal>
      </template>
    </ModalForm>
  </ModalDialog>
</template>

<script lang="ts" setup>
import ModalTitle from "@/js/contexts/shared/ui/components/modal/ModalTitle.vue";
import ModalHeader from "@/js/contexts/shared/ui/components/modal/ModalHeader.vue";
import ModalDialog from "@/js/contexts/shared/ui/components/modal/ModalDialog.vue";
import ModalForm from "@/js/contexts/shared/ui/components/modal/ModalForm.vue";
import FormSubmit from "@/js/contexts/form/ui/components/FormSubmit.vue";
import FormLabel from "@/js/contexts/form/ui/components/FormLabel.vue";
import FormInput from "@/js/contexts/form/ui/components/FormInput.vue";
import LinksOptionsForm from "@/js/contexts/links/ui/components/LinksOptionsForm.vue";
import AlertError from "@/js/contexts/shared/ui/components/alert/AlertError.vue";
import FormText from "@/js/contexts/form/ui/components/FormText.vue";
import useModal from "@/js/contexts/shared/composables/useModal";
import useForm from "@/js/contexts/shared/composables/useForm";
import type { LinkUpdate } from "@/js/contexts/shared/models/Link";
import useNotification from "@/js/contexts/shared/composables/useNotification";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import { Permission } from "@/js/contexts/shared/models/Permission";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import LinkDeleteModal from "@/js/contexts/links/ui/modals/LinkDeleteModal.vue";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import TrashCanRegularIcon from "@/svg/icons/trash-can-regular.svg?component";

type Response = {
  code: string;
  url: string;
  type: string;
  title: string | null;
  description: string | null;
  tags: string[] | null;
  domain_id: string | null;
  expires_at: string | null;
  has_password: boolean;
};

const emit = defineEmits<{
  fetch: [];
}>();

const { t } = useI18n();
const { hasPermission } = useAuth();
const { successNotification } = useNotification();
const updateModal = ref<InstanceType<typeof ModalDialog>>();
const modal = useModal(updateModal);
const route = useRoute();
const linkFetch = useFetch(`/api/links/${route.params.id}`, { isFetching: true });

const form = useForm<LinkUpdate & { has_password: boolean }>({
  code: "",
  url: "",
  type: "",
  title: null,
  description: null,
  tags: null,
  domain_id: null,
  expires_at: null,
  has_password: false,
});

onMounted(() => {
  linkFetch.get<Response>().then((response) => {
    form.values.code = response.data.code;
    form.values.url = response.data.url;
    form.values.type = response.data.type;
    form.values.title = response.data.title;
    form.values.description = response.data.description;
    form.values.tags = response.data.tags;
    form.values.domain_id = response.data.domain_id;
    form.values.expires_at = response.data.expires_at;
    form.values.has_password = response.data.has_password;
    form.updatedValues();
  });
});

const submit = () => {
  form.put(`/api/links/${route.params.id}`, {
    except: ["has_password"],
    onSuccess: () => {
      emit("fetch");
      modal.close();
      successNotification(t("app.links.update.success"));
    },
  });
};
const onDeleted = () => {
  emit("fetch");
  modal.close();
};
</script>
